from oscar.apps.dashboard.reports import utils
from django.core.exceptions import ImproperlyConfigured


class GeneratorRepository(utils.GeneratorRepository):

    @classmethod
    def register(cls, ReportGeneratorSubclass):
        if ReportGeneratorSubclass in cls.generators:
            raise ImproperlyConfigured('The class %s is already registered' % ReportGeneratorSubclass)
        cls.generators.append(ReportGeneratorSubclass)



from datetime import timedelta
from django.test import TestCase
from django.contrib.auth.models import User
from django.utils import timezone
from freezegun import freeze_time
from psycopg2.extras import DateTimeTZRange
from .. import models, tasks


@freeze_time('2019-10-03T12:00:00-04:00')
class ReportTest(TestCase):

    def setUp(self):
        self.staff_user = User.objects.create_user(
            username='root',
            is_staff=True)

        self.report = models.Report()
        self.report.uuid = 'd3c74a8b-e7ae-4482-bd9c-bee69fde5c5c'
        self.report.content_type = None
        self.report.owner = self.staff_user
        self.report.type_code = 'order_report'
        self.report.date_range = DateTimeTZRange(
            lower=(timezone.now() - timedelta(days=1)),
            upper=(timezone.now()))
        self.report.save()


    def test_generate_report(self):
        self.assertIsNone(self.report.started_on)
        self.assertIsNone(self.report.mime_type)
        self.assertIsNone(self.report.report_file.name)
        self.assertIsNone(self.report.completed_on)

        tasks.generate_report.apply_async(args=[self.report.uuid, 'CSV'])

        self.report.refresh_from_db()

        self.assertIsNotNone(self.report.started_on)
        self.assertEqual(self.report.mime_type, 'text/csv')
        self.assertTrue(self.report.report_file.name.startswith('oscar-reports/2019/10/03/d3c74a8b-e7ae-4482-bd9c-bee69fde5c5c'))
        self.assertTrue(self.report.report_file.name.endswith('.csv'))
        self.assertIsNotNone(self.report.completed_on)



from django.utils.translation import gettext_lazy as _
from django_tables2 import Column, TemplateColumn
from oscar.core.loading import get_class

DashboardTable = get_class('dashboard.tables', 'DashboardTable')


class ReportTable(DashboardTable):
    description = Column(accessor='description', orderable=False)
    date_range = TemplateColumn(
        template_name='oscar/dashboard/reports/report_row_date_range.html',
        verbose_name=_("Date Range"),
        orderable=False)

    owner = Column(accessor='owner.get_full_name', orderable=False, verbose_name=_('Owner'))
    status = Column(accessor='status_name', orderable=False, verbose_name=_('Report Status'))
    task_status = Column(accessor='celery_task_status', orderable=False, verbose_name=_('Task Status'))
    filesize = TemplateColumn(
        template_name='oscar/dashboard/reports/report_row_filesize.html',
        verbose_name=_("File Size"),
        orderable=False)

    queued_on = Column(accessor='queued_on')
    started_on = Column(accessor='started_on')
    completed_on = Column(accessor='completed_on')

    actions = TemplateColumn(
        template_name='oscar/dashboard/reports/report_row_actions.html',
        verbose_name=' ',
        orderable=False)

    class Meta(DashboardTable.Meta):
        pass



from django.test import TestCase
from .. import forms


class ReportFormTest(TestCase):
    def test_date_range_empty(self):
        form = forms.ReportForm(data={})
        self.assertFalse(form.is_valid())


    def test_date_range_complete(self):
        form = forms.ReportForm(data={
            'date_from': '2016-11-02',
            'date_to': '2016-11-03',
            'report_type': 'order_report'
        })
        self.assertTrue(form.is_valid())


    def test_date_range_incomplete(self):
        form = forms.ReportForm(data={
            'report_type': 'order_report',
            'date_to': '',
            'date_from': '2016-11-02'
        })
        self.assertTrue(form.is_valid(), form.errors)
        form = forms.ReportForm(data={
            'date_from': '',
            'date_to': '2016-11-03',
            'report_type': 'order_report'
        })
        self.assertTrue(form.is_valid())
        form = forms.ReportForm(data={
            'date_from': '2016-11-02',
            'report_type': 'order_report'
        })
        self.assertTrue(form.is_valid())
        form = forms.ReportForm(data={
            'date_to': '2016-11-03',
            'report_type': 'order_report'
        })
        self.assertTrue(form.is_valid())


    def test_date_range_incorrect(self):
        form = forms.ReportForm(data={
            'date_from': '2016-11-03',
            'date_to': '2016-11-02',
            'report_type': 'order_report'
        })
        self.assertFalse(form.is_valid())



from django.conf.urls import url
from oscar.apps.dashboard.reports import apps
from oscar.core.loading import get_class


class ReportsDashboardConfig(apps.ReportsDashboardConfig):
    name = 'oscarreports'


    def ready(self):
        super().ready()
        from . import handlers  # NOQA
        self.index_view = get_class('reports_dashboard.views', 'IndexView')
        self.download_view = get_class('reports_dashboard.views', 'ReportDownloadView')
        self.delete_view = get_class('reports_dashboard.views', 'ReportDeleteView')


    def get_urls(self):
        urls = [
            url(r'^$',
                self.index_view.as_view(), name='reports-index'),
            url(r'^(?P<uuid>[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12})/download/$',
                self.download_view.as_view(), name='reports-download'),
            url(r'^(?P<uuid>[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12})/delete/$',
                self.delete_view.as_view(), name='reports-delete'),
        ]
        return self.post_process_urls(urls)



from django.db.models.signals import pre_delete
from django.dispatch import receiver
from .models import Report


@receiver(pre_delete, sender=Report)
def auto_delete_report_file(sender, instance, **kwargs):
    if instance.report_file:
        instance.report_file.delete(save=False)



from django.urls import reverse_lazy
from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from django.shortcuts import redirect
from django.views.generic.edit import FormMixin, DeleteView
from django.views.generic.detail import BaseDetailView
from django.http import FileResponse, Http404
from django_tables2 import SingleTableView
from psycopg2.extras import DateTimeTZRange
from .models import Report
from .tables import ReportTable
from .forms import ReportForm
import os.path


class IndexView(FormMixin, SingleTableView):
    template_name = 'oscar/dashboard/reports/index.html'
    table_pagination = True
    model = Report
    table_class = ReportTable
    context_table_name = 'reports'
    desc_template = ''
    description = ''

    form_class = ReportForm

    def dispatch(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        return super().dispatch(request, *args, **kwargs)


    def get_queryset(self):
        queryset = self.model.objects.all().order_by('-queued_on')
        return queryset


    def get_table_pagination(self, table):
        return dict(per_page=20)


    def post(self, request, *args, **kwargs):
        if self.form.is_valid():
            # Create report
            report = Report()
            report.content_type = None
            report.owner = request.user
            report.type_code = self.form.cleaned_data['report_type']
            report.date_range = DateTimeTZRange(
                lower=self.form.cleaned_data['date_from'],
                upper=self.form.cleaned_data['date_to'])
            report.save()
            # Queue report
            report.queue()
            # Message user
            messages.info(request, _("Successfully queued report for generation"))
            return redirect('dashboard:reports-index')
        return self.get(request, *args, **kwargs)


    def get_table(self, **kwargs):
        table = super().get_table(**kwargs)
        table.caption = _("Reports")
        return table


    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['form'] = self.form
        return context


class ReportDownloadView(BaseDetailView):
    model = Report
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def render_to_response(self, context, **response_kwargs):
        report = context['object']
        if not report.report_file.name:
            raise Http404()
        # Build filename
        extension = os.path.splitext(report.report_file.name)[1].replace('.', '')
        filename = '{date}_{type_code}_{uuid}.{ext}'.format(
            date=report.created_on.strftime('%Y-%m-%d'),
            type_code=report.type_code,
            uuid=report.uuid,
            ext=extension)
        # Send file to client
        resp = FileResponse(report.report_file,
             as_attachment=True,
             filename=filename)
        resp['Content-Type'] = report.mime_type
        return resp


class ReportDeleteView(DeleteView):
    model = Report
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'
    success_url = reverse_lazy('dashboard:reports-index')
    template_name = 'oscar/dashboard/reports/report_confirm_delete.html'
